/* Nazwa pliku: ExtremeC_examples_chapter12_3.c
 * Opis: Obsługa Unicode.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef __APPLE__

#include <stdint.h>

typedef uint16_t char16_t;
typedef uint32_t char32_t;

#else
#include <uchar.h> // Niezbędne dla typów char16_t i char32_t.
#endif

typedef struct {
  long num_chars;
  long num_bytes;
} unicode_len_t;

unicode_len_t strlen_ascii(char* str) {
  unicode_len_t res;
  res.num_chars = 0;
  res.num_bytes = 0;
  if (!str) {
    return res;
  }
  res.num_chars = strlen(str) + 1;
  res.num_bytes = strlen(str) + 1;
  return res;
}

unicode_len_t strlen_u8(char* str) {
  unicode_len_t res;
  res.num_chars = 0;
  res.num_bytes = 0;
  if (!str) {
    return res;
  }
  // Ostatni znak null.
  res.num_chars = 1;
  res.num_bytes = 1;
  while (*str) {
    if ((*str | 0x7f) == 0x7f) { // 0x7f = 0b01111111.
      res.num_chars++;
      res.num_bytes++;
      str++;
    } else if ((*str & 0xf0) == 0xf0) { // 0xf0 = 0b11110000.
      res.num_chars++;
      res.num_bytes += 4;
      str += 4;
    } else if ((*str & 0xe0) == 0xe0) { // 0xe0 = 0b11100000.
      res.num_chars++;
      res.num_bytes += 3;
      str += 3;
    } else if ((*str & 0xc0) == 0xc0) { // 0xc0 = 0b11000000.
      res.num_chars++;
      res.num_bytes += 2;
      str += 2;
    } else {
      fprintf(stderr, "Ciąg tekstowy UTF-8 jest nieprawidłowy!\n");
      exit(1);
    }
  }
  return res;
}

unicode_len_t strlen_u16(char16_t* str) {
  unicode_len_t res;
  res.num_chars = 0;
  res.num_bytes = 0;
  if (!str) {
    return res;
  }
  // Ostatni znak null.
  res.num_chars = 1;
  res.num_bytes = 2;
  while (*str) {
    if (*str < 0xdc00 || *str > 0xdfff) {
      res.num_chars++;
      res.num_bytes += 2;
      str++;
    } else {
      res.num_chars++;
      res.num_bytes += 4;
      str += 2;
    }
  }
  return res;
}
unicode_len_t strlen_u32(char32_t* str) {
  unicode_len_t res;
  res.num_chars = 0;
  res.num_bytes = 0;
  if (!str) {
    return res;
  }
  // Ostatni znak null.
  res.num_chars = 1;
  res.num_bytes = 4;
  while (*str) {
      res.num_chars++;
      res.num_bytes += 4;
      str++;
  }
  return res;
}

int main(int argc, char** argv) {

  char ascii_string[32] = "Witaj, świecie!";

  char utf8_string[32] = u8"Witaj, świecie!";
  char utf8_string_2[32] = u8"درود دنیا!";

  char16_t utf16_string[32] = u"Witaj, świecie!";
  char16_t utf16_string_2[32] = u"درود دنیا!";
  char16_t utf16_string_3[32] = u"𝦵হ𝡩!";

  char32_t utf32_string[32] = U"Witaj, świecie!";
  char32_t utf32_string_2[32] = U"درود دنیا!";
  char32_t utf32_string_3[32] = U"𝦵হ𝡩!";

  unicode_len_t len = strlen_ascii(ascii_string);
  printf("Długość ciągu tekstowego ASCII:\t\t\t %ld znaków, %ld bajtów\n\n",
      len.num_chars, len.num_bytes);

  len = strlen_u8(utf8_string);
  printf("Długość ciągu tekstowego UTF-8 w języku polskim:\t\t %ld znaków, %ld bajtów\n",
      len.num_chars, len.num_bytes);
  len = strlen_u16(utf16_string);
  printf("Długość ciągu tekstowego UTF-16 w języku polskim:\t %ld znaków, %ld bajtów\n",
      len.num_chars, len.num_bytes);
  len = strlen_u32(utf32_string);
  printf("Długość ciągu tekstowego UTF-32 w języku polskim:\t %ld znaków, %ld bajtów\n\n",
      len.num_chars, len.num_bytes);

  len = strlen_u8(utf8_string_2);
  printf("Długość ciągu tekstowego UTF-8 w języku perskim:\t\t %ld znaków, %ld bajtów\n",
      len.num_chars, len.num_bytes);
  len = strlen_u16(utf16_string_2);
  printf("Długość ciągu tekstowego UTF-16 w języku perskim:\t %ld znaków, %ld bajtów\n",
      len.num_chars, len.num_bytes);
  len = strlen_u32(utf32_string_2);
  printf("Długość ciągu tekstowego UTF-32 w języku perskim:\t %ld znaków, %ld bajtów\n\n",
      len.num_chars, len.num_bytes);

  len = strlen_u16(utf16_string_3);
  printf("Długość ciągu tekstowego UTF-16 w języku obcym:\t\t %ld znaków, %ld bajtów\n",
      len.num_chars, len.num_bytes);
  len = strlen_u32(utf32_string_3);
  printf("Długość ciągu tekstowego UTF-32 w języku obcym:\t\t %ld znaków, %ld bajtów\n",
      len.num_chars, len.num_bytes);

  return 0;
}
